
# This file is part of FatxFast.
#
#    FatxFast is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    FatxFast is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with FatxFast.  If not, see <http://www.gnu.org/licenses/>.

# Versioning based on: 
# http://en.wikipedia.org/wiki/Versioning#Designating_development_stage
__author__ = "dryatu (c) 2013"
__version__ = "1.1.2"

import random
from FatxFast.objects import entity
from FatxFast.constants import *
from FatxFast import camera

class SpawnArea(entity.GameEntity):
    
    def __init__(self, world, **kwargs):
        super(SpawnArea, self).__init__(entity_id=SPAWNAREA_ID, **kwargs)
        self.world = world
        self.spawntimer = int(kwargs.get("spawntimer", 0))
        self.maxspawns = int(kwargs.get("max_spawns", 1))
        if "entity_count" in kwargs:
            self.entity_count = int(kwargs["entity_count"])
        else:
            self.entity_count = None
        self.entity_classes = set(kwargs.get("entity_classes", []))
        self.spawntime = int(kwargs.get("spawntime", 5000))
        self.entities = camera.CameraGroup()

    def add_entity_class(self, classid):
        self.entity_classes.add(classid)

    def create_entity(self):
        entity = self.get_random_entity()
        self.world.add(entity)
        self.entities.add(entity)
        self.spawntimer = 0 

    def get_random_entity(self):
        entity_class = random.sample(self.entity_classes, 1)[0]
        entity = entity_class()
        random_x = random.randint(self.rect.x, self.rect.right)
        random_y = random.randint(self.rect.y, self.rect.bottom)
        entity.rect.centerx, entity.rect.centery = random_x, random_y
        return entity

    def spawn_everything(self):
        if self.entity_count is None:
            to_spawn = self.maxspawns
        else:
            to_spawn = self.entity_count
        for idx in xrange(to_spawn):
            self.create_entity()

    def update(self, dt):
        self.spawntimer += dt
        if self.entity_count > len(self.entities):
            self.spawntimer = 0            
        if (self.spawntimer >= self.spawntime 
            and self.entity_count < self.maxspawns):
            self.create_entity()
        self.entity_count = len(self.entities)
